# TEXTRACT -- Extract TRES image into one or two echelle files
# By Doug Mink, SAO Telescope Data Center
# September 24, 2007

procedure textract (images)

	file	images=""	{prompt="Input image(s) to extract"}
	string	fibsize="s"	{prompt="Fiber size (s m l)"}
	int	fibs=12	{prompt="Fibers illuminated (12 13 3)"}

	file	aproot=""	{prompt="Root filename for aperature reference spectrum"}
	file	apdir=""	{prompt="Directory for reference spectra"}
	string	readnoise="1"	{prompt="Read out noise sigma (photons)"}
	string	gain="1"	{prompt="Photon gain (photons/data number)"}
	real    npeaks=0.2	{prompt="Select brightest peaks"}
	bool	interactive=no	{prompt="Run apall interactively (y or n)"}
	bool	review=no	{prompt="Review results of extraction (y or n)"}
	bool	deltemp=yes	{prompt="Delete temporary file (y or n)"}
	bool	verbose=yes	{prompt="Log progress (y or n)"}
	bool	debug=yes	{prompt="Print debugging information (y or n)"}

	begin
	bool vb, db
	file imin, imfib1, imfib2, imfib3, imroot, templist, listfile
	file apfib1, apfib2, apfib3, adir, cwd
	file apreffib2, apreffib1, apreffib3
	file apfib1path, apfib2path, apfib3path
	file apfib1new, apfib2new, apfib3new
	file imfib1path, imfib2path, imfib3path, outfib1, outfib2, outfib3
	int lroot
	string history, ims, tchar, args

	ims = images
	adir = apdir
	fixdir (adir)
	adir = fixdir.newdir
	vb = verbose
	db = debug


# Copy existing aperture maps to local database/ directory
	if (!access ("database/"))
	    mkdir ("database")
	pathname | scan (cwd)

# Get file names for fiber 1 reference image and aperture file
	apreffib1 = ""
	if (fibs == 12 || fibs == 13) {
	    apreffib1 = aproot // fibsize // "1"
	    smatch (apreffib1, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		apfib1 = substr (apreffib1, 1, lroot)
		apfib1 = "ap" // apfib1
		imfib1 = apreffib1
		}
	    else {
		imfib1 = apreffib1 // ".fits"
		apfib1 = "ap" // apreffib1
		}

# Copy fiber 1 reference aperture file to local database/
	    apfib1path = adir // "database/" // apfib1
	    apfib1new = "database/" // apfib1
	    if (!access (apfib1new)) {
		copy (apfib1path, apfib1new)
		print ("TEXTRACT: ",apfib1path," => ", apfib1new)
		}
	    else
		print ("TEXTRACT: ",apfib1new," already exists")

# Copy fiber 1 reference image file to local directory
	    imfib1path = adir // imfib1
	    if (!access (imfib1)) {
		print ("TEXTRACT: Copying ",imfib1path, " to ",cwd)
		args = osfn (imfib1path) // " " // imfib1
		lns (args)
		}
	    else
		print ("TEXTRACT: ",imfib1," already exists")
	    }

# Get file names for fiber 2 reference image and aperture file
	apreffib2 = ""
	if (fibs == 12) {
	    apreffib2 = aproot // fibsize // "2"
	    smatch (apreffib2, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		apfib2 = substr (apreffib2, 1, lroot)
		apfib2 = "ap" // apfib2
		imfib2 = apreffib2
		}
	    else {
		imfib2 = apreffib2 // ".fits"
		apfib2 = "ap" // apreffib2
		}

# Copy fiber 2 reference aperture file to local database/
	    apfib2path = adir // "database/" // apfib2
	    apfib2new = "database/" // apfib2
	    if (!access (apfib2new)) {
		print ("TEXTRACT: ",apfib2path," => ", apfib2new)
		copy (apfib2path, apfib2new)
		}
	    else
		print ("TEXTRACT: ",apfib2new," already exists")

# Copy fiber 2 reference image file to local directory
	    imfib2path = adir // imfib2
	    if (!access (imfib2)) {
		print ("TEXTRACT: Copying ",imfib2path, " to ",cwd)
		args = osfn (imfib2path) // " " // imfib2
		lns (args)
		}
	    else
		print ("TEXTRACT: ",imfib2," already exists")
	    }

# Get file names for fiber 3 calibration reference image and aperture file
	if (fibs == 13 || fibs == 3) {
	    apreffib3 = aproot // fibsize // "3"
	    smatch (apreffib3, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		apfib3 = substr (apreffib3, 1, lroot)
		apfib3 = "ap" // apfib3
		imfib3 = apreffib3
		}
	    else {
		    imfib3 = apreffib3 // ".fits"
		apfib3 = "ap" // apreffib3
		}

# Copy fiber 3 calbibration reference aperture file to local database/
	    apfib3path = adir // "database/" // apfib3
	    apfib3new = "database/" // apfib3
	    if (!access (apfib3new)) {
		print ("TEXTRACT: ",apfib3path," => ", apfib3new)
		copy (apfib3path, apfib3new)
		}
	    else
		print ("TEXTRACT: ",apfib3new," already exists")

# Copy fiber 3 calibration reference image file to local directory
	    imfib3path = adir // imfib3
	    if (!access (imfib3)) {
		print ("TEXTRACT: Copying ",imfib3path, " to ",cwd)
		args = osfn (imfib3path) // " " // imfib3
		lns (args)
		}
	    else
		print ("TEXTRACT: ",imfib3," already exists")
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("tempexlist")
        if (tchar == '@') {
	    listfile = substr (ims,2,100)
	    copy (listfile, templist)
	    }
	else {
	    files (ims, >templist)
	    }
	list = templist
	while (fscan (list,imin) != EOF) {

# Make output file name
	    smatch (imin, ".ec")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		}
	    else {
		smatch (imin, ".fits")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    imroot = substr (imin, 1, lroot)
		    }
		else {
		    imroot = imin
		    }
		}
	    imin = imroot // ".fits"
	    outfib1 = imroot // "_fib1.ec.fits"
	    outfib2 = imroot // "_fib2.ec.fits"
	    outfib3 = imroot // "_fib3.ec.fits"

# Make sure this is not already an extracted image
	    imgetp.default = "0"
	    imgetp (imin, "TEXTRACT")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("TEXTRACT: ", imin, " is already an extracted image")
		    print ("TEXTRACT = ", imgetp.value)
		    }
		next
		}
	    if (db) {
		if (apreffib1 != "" && apreffib2 != "")
		    print ("TEXTRACT: ", imin, " -> ", outfib1," and ", outfib2)
		else if (apreffib1 != "" && apreffib3 != "")
		    print ("TEXTRACT: ", imin, " -> ", outfib1," and ", outfib3)
		else if (apreffib1 != "")
		    print ("TEXTRACT: ", imin, " -> ", outfib1)
		else if (apreffib2 != "")
		    print ("TEXTRACT: ", imin, " -> ", outfib2)
		else if (apreffib3 != "")
		    print ("TEXTRACT: ", imin, " -> ", outfib3)
		}

# Set up apall parameters

	    apall.apertures = "1-52"
	    apall.format = "echelle"
	    apall.profiles = ""
	    apall.interactive = interactive
	    apall.find = no
	    apall.recenter = yes
	    apall.resize = no
	    apall.edit = no
	    apall.trace = no
	    apall.fittrace = no
	    apall.extract = yes
	    apall.extras = no
	    apall.review = review
	    apall.line = INDEF
	    apall.nsum = 10
	    apall.lower = -5.
	    apall.upper = 5.
	    apall.apidtable = ""
	    apall.b_function = "legendre"
	    apall.b_order = 1
	    apall.b_sample = "-10:-6,6:10"
	    apall.b_naverage = -3
	    apall.b_niterate = 3
	    apall.b_low_reject = 3.
	    apall.b_high_reject = 3.
	    apall.b_grow = 0.
	    apall.width = 5.
	    apall.radius = 10.
	    apall.threshold = 0.
	    apall.minsep = 5.
	    apall.maxsep = 1000.
	    apall.order = "increasing"
	    apall.aprecenter = ""
	    apall.npeaks = 20
	    apall.shift = yes
	    apall.llimit = INDEF
	    apall.ulimit = INDEF
	    apall.ylevel = 0.1
	    apall.peak = yes
	    apall.bkg = yes
	    apall.r_grow = 0.
	    apall.avglimits = no
	    apall.t_nsum = 10
	    apall.t_step = 10
	    apall.t_nlost = 3
	    apall.t_function = "legendre"
	    apall.t_order = 3
	    apall.t_sample = "*"
	    apall.t_naverage = 1
	    apall.t_niterate = 0
	    apall.t_low_reject = 3.
	    apall.t_high_reject = 3.
	    apall.t_grow = 0.
	    apall.background = "none"
	    apall.skybox = 1
	    apall.weights = "none"
	    apall.pfit = "fit1d"
	    apall.clean = no
	    apall.saturation = INDEF
	    apall.readnoise = "0."
	    apall.gain = "1."
	    apall.lsigma = 4.
	    apall.usigma = 4.
	    apall.nsubaps = 1

# Set time of operation
	    gettime ()
	    history = gettime.date // " " // gettime.time // " extracted from " // imin
	    hedit.show = vb

# Extract fiber 1 object (usually) spectrum to echelle file
	    if (apreffib1 != "") {
		apall.references = imfib1
		print ("TEXTRACT: Extracting ",outfib1)
		apall (imin, output=outfib1)
		hedit (outfib1, "BANDID2", "echelle order", add+, verify-)
		hedit (outfib1, "TEXTRACT", history, add+, verify-)
		hedit (outfib1, "TEXAPREF", imfib1path, add+, verify-)
		}

# Extract fiber 2 (usually) background spectrum to echelle file
	    if (apreffib2 != "") {
		apall.references = imfib2
		print ("TEXTRACT: Extracting ",outfib2)
		apall (imin, output=outfib2)
		hedit (outfib2, "BANDID2", "echelle order", add+, verify-)
		hedit (outfib2, "TEXTRACT", history, add+, verify-)
		hedit (outfib2, "TEXAPREF", imfib2path, add+, verify-)
		}

# Extract fiber 3 calibration spectrum to echelle file
	    if (apreffib3 != "") {
		apall.references = imfib3
		print ("TEXTRACT: Extracting ",outfib3)
		apall (imin, output=outfib3)
		hedit (outfib3, "BANDID2", "echelle order", add+, verify-)
		hedit (outfib3, "TEXTRACT", history, add+, verify-)
		hedit (outfib3, "TEXAPREF", imfib3path, add+, verify-)
		}
	    }

	if (deltemp)
	    delete (templist)

end

# Sep 13 2007	New task based on hectospec.hextract
# Sep 18 2007	Allow extraction of any of the three apertures of data
# Sep 21 2007	Use apall instead of apsum and recenter
# Sep 24 2007	Change aperture templates names to root||apsize||apnum
